from Globals import package_home
from Products.CMFCore.DirectoryView import addDirectoryViews, registerDirectory
from Products.CMFCore.DirectoryView import manage_listAvailableDirectories
from Products.CMFCore.utils import getToolByName, minimalpath
from OFS.ObjectManager import BadRequestException
from StringIO import StringIO
from os.path import isdir, join
from os import listdir

from Products.PloneJSOrder.config import *


def install(self):
    out = StringIO()
    installSkins(self, out, packageGlobals)
    installJavaScripts(self, out, ('jsorder.js',))
    enableJavaScripts(self, out, ('vcXMLRPC.js',))
    print >> out, "Successfully installed %s." % packageName
    return out.getvalue()


def uninstall(self):
    out = StringIO()
    uninstallJavaScripts(self, out, ('jsorder.js',))
    print >> out, "Successfully uninstalled %s." % packageName
    return out.getvalue()


def enableJavaScripts(self, out, scripts):
    """ enable the given javascripts within plone registry tool """
    jsreg = getToolByName(self, 'portal_javascripts', None)
    if jsreg:
        for name in scripts:
            res = jsreg.getResource(name)
	    if not res.getEnabled():
	        res.setEnabled(True)
            print >> out, "> Enabled javascript '%s'." % name
	jsreg.cookResources()		# re-cook all js resources


def installJavaScripts(self, out, scripts):
    """ register javascripts with plone registry tool """
    jsreg = getToolByName(self, 'portal_javascripts', None)
    if jsreg:
        resourceIds = jsreg.getResourceIds()
        for name in scripts:
            if not name in resourceIds:
                jsreg.registerScript(name)
                print >> out, "> Registered javascript '%s'." % name


def uninstallJavaScripts(self, out, scripts):
    """ unregister javascripts with plone registry tool """
    jsreg = getToolByName(self, 'portal_javascripts', None)
    if jsreg:
        resourceIds = jsreg.getResourceIds()
        for name in scripts:
            if name in resourceIds:
                jsreg.unregisterResource(name)
                print >> out, "> Unregistered javascript '%s'." % name


def installSkins(self, out, globals, product_skins_dir='skins'):
    """ install a filesystem directory view into the skins directory """
    skinstool = getToolByName(self, 'portal_skins')
    fullProductSkinsPath = join(package_home(globals), product_skins_dir)
    productSkinsPath = minimalpath(fullProductSkinsPath)

    registeredDirectories = manage_listAvailableDirectories()
    if productSkinsPath not in registeredDirectories:
        try:
            registerDirectory(product_skins_dir, globals)
            print >> out, "> Registered skin '%s'." % product_skins_dir
        except OSError, exc:
            if exc.errno == 2:		# No such file or directory
                return
            raise

    try:
        addDirectoryViews(skinstool, product_skins_dir, globals)
    except BadRequestException, e:
        if str(e).endswith(' is reserved.'):
            raise
        pass				# directory view has already been added

    for productSkinName in listdir(fullProductSkinsPath):
        if productSkinName.find('.') != -1 or productSkinName in ('CVS', '{arch}'):
            continue
        if isdir(join(fullProductSkinsPath, productSkinName)):
            for skinName in skinstool.getSkinSelections():
                path = skinstool.getSkinPath(skinName)
                path = [i.strip() for i in path.split(',')]
                if productSkinName not in path:
                    path.append(productSkinName)
                    print >> out, "> Added '%s' to skin '%s'." % (productSkinName, skinName)
                path = ', '.join(path)
                skinstool.addSkinSelection(skinName, path)

